﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

public class Battery : MonoBehaviour
{
    public float CurrentBatteryLife = 100.0f;
    public float MaxBatteryLife = 100.0f;
    public float TotalDrainSeconds = 60.0f;
    public UnityEvent OnBatteryEmpty;
    public bool PauseDrain = false;
    public Coroutine _Coroutine;

    private float drain;
    private GameManager gameManager;

    // Start is called before the first frame update
    void Start()
    {
        gameManager = GameManager.Instance;
        drain = MaxBatteryLife / TotalDrainSeconds;

        gameManager.OnChangeStateToStart.AddListener(OnEnable);
        gameManager.OnChangeStateToEndAndWin.AddListener(OnEndStage);
    }

    private void OnEnable()
    {
        if(gameManager != null && gameManager.CurState == GameState.Start)
        {
            if (_Coroutine != null) StopCoroutine(_Coroutine);
            _Coroutine = StartCoroutine(DrainBattery());
        }
    }

    IEnumerator DrainBattery()
    {
        while (true)
        {
            yield return new WaitForSeconds(1.0f);

            if (PauseDrain != true)
            {
                Debug.Log("Battery Drain: " + drain);
                CurrentBatteryLife -= drain;
            }

            if(CurrentBatteryLife <= 0)
            {
                CurrentBatteryLife = 0;
                OnBatteryEmpty.Invoke();
                yield break;
            }
        }
    }

    public void OnEndStage()
    {
        PauseDrain = true;
    }

    public float GetNormalizeBatteryLife()
    {
        return CurrentBatteryLife / MaxBatteryLife;
    }
}
