﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

public enum BuildingState
{
    Show,
    Hide
}

public class Building : MonoBehaviour
{
    const float PER_ALPHA = 0.05f;
    const float MAX_ALPHA = 1.0f;

    public BuildingState CurBuildingState = BuildingState.Show;
    MeshRenderer meshRenderer;
    private float alpha = 1f;
    public bool onTransparencyStart = false;

    // Start is called before the first frame update
    void Start()
    {
        meshRenderer = GetComponent<MeshRenderer>();
    }

    public void SetTransparent(bool condition)
    {
        if (onTransparencyStart == true) return;
        onTransparencyStart = true;

        if (condition == true)
        {
            StartCoroutine("ReduceAlpha");
        }
        else
        {
            StartCoroutine("GainAlpha");
        }
    }

    IEnumerator ReduceAlpha()
    {
        while (true)
        {
            if (alpha > PER_ALPHA)
            {
                alpha -= PER_ALPHA;

                SetAlpha(alpha);
            }
            else
            {
                onTransparencyStart = false;
                CurBuildingState = BuildingState.Hide;
                yield break;
            }

            yield return new WaitForEndOfFrame();
        }
    }

    IEnumerator GainAlpha()
    {
        while (true)
        {
            if (alpha < MAX_ALPHA)
            {
                alpha += PER_ALPHA;

                SetAlpha(alpha);
            }
            else
            {
                onTransparencyStart = false;
                CurBuildingState = BuildingState.Show;
                yield break;
            }

            yield return new WaitForEndOfFrame();
        }
    }

    void SetAlpha(float alpha)
    {
        Color color = meshRenderer.material.color;

        meshRenderer.material.color = new Color(color.r, color.g, color.b, alpha);
    }
}
