﻿using System.Linq;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.Events;

public class DialogueHandler : MonoBehaviour
{
    public List<string> Dialogues;
    public Text DialogueText;
    public GameObject DialogueBox;
    public UnityEvent OnDialogueFinish;
    public GameObject Phone;
    public GameObject Dialogue;
    public GameObject Dialogue2;
    public bool Additional = false;
    public int IndexToInsert;

    private string currentText = "";
    private Coroutine curCoroutine;
    private int index = 0;
    private readonly float delay = 0.1f;
    private string previousText;
    private GameManager gameManager;
    private PlayerController playerController;

    private void OnEnable()
    {
        gameManager = GameManager.Instance;
        if (gameManager.CurState != GameState.End)
        {
            gameManager.CurState = GameState.None;
        }
        playerController = PlayerController.Instance;

        if(DialogueBox != null) DialogueBox.SetActive(false);

        DialogueBox = Dialogue;
        DialogueText = DialogueBox.GetComponentInChildren<Text>();
        
        DialogueBox.SetActive(true);

        StartText();
    }

    // Update is called once per frame
    void Update()
    {
        if(Input.GetKeyDown(KeyCode.Return))
        {
            if (index < Dialogues.Count)
            {
                if(currentText == previousText)
                {
                    if (index == IndexToInsert && Additional == true)
                    {
                        if (Dialogue2 != null)
                        {
                            DialogueBox.SetActive(false);
                            DialogueBox = Dialogue2;
                            DialogueBox.SetActive(true);

                            DialogueText = DialogueBox.GetComponentInChildren<Text>();
                            DialogueText.text = previousText;
                        }
                    }

                    StartText();
                }
                else if (currentText.Length != previousText.Length)
                {
                    StopCoroutine(curCoroutine);
                    currentText = previousText;
                    DialogueText.text = currentText;

                    if (index == IndexToInsert && Additional == true)
                    {
                        if (Phone != null)
                        {
                            Phone.SetActive(true);
                        }
                    }
                    index++;
                }
            }
            else
            {
                //Game Start
                DialogueBox.SetActive(false);
                playerController.enabled = true;

                if (gameManager.CurState != GameState.End) gameManager.CurState = GameState.Start;
                
                OnDialogueFinish.Invoke();
            }
        }
    }

    void StartText()
    {
        curCoroutine = StartCoroutine(TypeWriter(Dialogues[index]));
    }

    IEnumerator TypeWriter(string text)
    {
        previousText = text;
        for (int i = 0; i < previousText.Length; i++)
        {
            currentText = previousText.Substring(0, i + 1);
            DialogueText.text = currentText;
            yield return new WaitForSeconds(delay);
        }
        if (index == IndexToInsert && Additional == true)
        {
            if (Phone != null)
            {
                Phone.SetActive(true);
            }
        }
        index++;
    }

    public void StartDialogue()
    {
        this.enabled = true;
    }
}
