﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

public class EnterMechanic : MonoBehaviour
{
    public KeyCode SpaceKey = KeyCode.Space;
    public float Timer = 2.0f;
    public UnityEvent OnEnterEvent;
    public UnityEvent OnExitEvent;

    public GameObject Target;
    private PlayerController pController;
    private File file;
    private Wifi wifi;
    private Coroutine enterCoroutine = null;
    private Coroutine wifiCoroutine = null;
    private GameManager gameManager;

    // Start is called before the first frame update
    void Start()
    {
        wifi = GetComponent<Wifi>();
        gameManager = GameManager.Instance;
    }

    // Update is called once per frame
    void Update()
    {
        if(gameManager != null && gameManager.CurState == GameState.End) return;
        
        if (Input.GetKeyDown(SpaceKey) && Target != null && pController.CurState == WorldState.Wander)
        {
            enterCoroutine = StartCoroutine(Enter());
        }
    }

    IEnumerator Enter()
    {
        EnterEffect();
        yield return new WaitForSeconds(Timer);
        ExitEffect();
    }

    void EnterEffect()
    {
        SetTargetActiveTo(false);
        pController.CurState = WorldState.Interact;

        if (wifi != null)
        {
            wifiCoroutine = StartCoroutine(wifi.StartDownload(file));
        }
        else Debug.Log("No WIFI");

        OnEnterEvent.Invoke();
    }

    void ExitEffect()
    {
        SetTargetActiveTo(true);
        pController.CurState = WorldState.Wander;

        if (wifi != null && wifiCoroutine != null)
        {
            StopCoroutine(wifiCoroutine);
            Destroy(wifi);
        }
        else Debug.Log("No WIFI");

        if (enterCoroutine != null) StopCoroutine(enterCoroutine);

        OnExitEvent.Invoke();
    }

    void OnTriggerEnter(Collider collider)
    {
        PlayerController pc = collider.gameObject.GetComponent<PlayerController>();
        if (pc != null)
        {
            Target = collider.gameObject;
            pController = pc;
            file = Target.GetComponent<File>();
            file.OnFileDownloaded.AddListener(ExitEffect);
        }
    }

    void OnTriggerExit(Collider collider)
    {
        if (collider.gameObject.GetComponent<PlayerController>())
        {
            Target = null;
            pController = null;
            file.OnFileDownloaded.RemoveListener(ExitEffect);
            file = null;
        }
    }

    void SetTargetActiveTo(bool condition)
    {
        Target.GetComponent<PlayerController>().enabled = condition;
        //Target.GetComponent<MeshRenderer>().enabled = condition;

        foreach (Transform child in Target.transform)
        {
            MeshRenderer mr = child.GetComponent<MeshRenderer>();
            child.gameObject.SetActive(condition);

            if (mr != null)
            {
                mr.enabled = condition;
            }
        }
    }
}
