﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

public enum GameState
{
    None,
    Start,
    End
}

public enum WinCondition
{
    None,
    Win,
    Lose
}

public class GameManager : MonoBehaviour
{
    public static GameManager Instance;

    public Transform Player;
    public GameState CurState = GameState.None;
    public WinCondition CurCondition = WinCondition.None;
    public UnityEvent OnChangeStateToStart;
    public UnityEvent OnChangeStateToEndAndWin;
    public UnityEvent OnChangeStateToEndAndLose;

    public bool UseTimeStart = true;
    public float Timer = 1.0f;
    File file;
    Battery battery;

    private void Awake()
    {
        if (Instance == null) Instance = this;
    }

    // Start is called before the first frame update
    void Start()
    {
        file = Player.GetComponent<File>();
        battery = Player.GetComponent<Battery>();

        //Add Listener
        file.OnFileDownloaded.AddListener(FileDownloadFinish);
        battery.OnBatteryEmpty.AddListener(BatteryDrainEmpty);

        if(UseTimeStart == true)
        {
            StartCoroutine(OnStart());
        }
    }

    IEnumerator OnStart()
    {
        yield return new WaitForSeconds(Timer);

        CurState = GameState.Start;
        OnChangeStateToStart.Invoke();
    }

    void FileDownloadFinish()
    {
        Debug.Log("Game End. You Win.");
        CurState = GameState.End;
        CurCondition = WinCondition.Win;

        OnChangeStateToEndAndWin.Invoke();
    }

    void BatteryDrainEmpty()
    {
        Debug.Log("Game End. You Lose.");
        CurState = GameState.End;
        CurCondition = WinCondition.Lose;

        OnChangeStateToEndAndLose.Invoke();
    }
}
