﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public enum WorldState
{
    Wander,
    Interact
}

public class PlayerController : MonoBehaviour
{
    public static PlayerController Instance;

    public WorldState CurState = WorldState.Wander;

    [Header("Movements")]
    public KeyCode ForwardKey = KeyCode.UpArrow;
    public KeyCode BackKey = KeyCode.DownArrow;
    public KeyCode LeftKey = KeyCode.LeftArrow;
    public KeyCode RightKey = KeyCode.RightArrow;

    private Vector3 faceFront = new Vector3(0, 0, 0);
    private Vector3 faceBack = new Vector3(0, 180, 0);
    private Vector3 faceLeft = new Vector3(0, -90, 0);
    private Vector3 faceRight = new Vector3(0, 90, 0);
    private readonly float step = 4.0f;
    private Vector3 rot = Vector3.zero;
    private GameManager gameManager;
    private Rigidbody rb;

    private void Awake()
    {
        if(Instance == null)
        {
            Instance = this;
        }
        else if(Instance != this)
        {
            Destroy(this);
        }
    }

    private void Start()
    {
        gameManager = GameManager.Instance;
        rb = GetComponent<Rigidbody>();
    }

    // Update is called once per frame
    void FixedUpdate()
    {
        if (gameManager.CurState != GameState.Start) return;

        Movement();
        if (rb.velocity != Vector3.zero) rb.velocity = Vector3.zero;
    }

    void Movement()
    {
        float speed = step * Time.deltaTime;
        Vector3 dir = Vector3.zero;

        if (Input.GetKey(ForwardKey))
        {
            dir += transform.forward * speed;
            rot = faceFront;
        }
        if(Input.GetKey(BackKey))
        {
            dir -= transform.forward * speed;
            rot = faceBack; 
        }
        if (Input.GetKey(LeftKey))
        {
            dir += transform.right * speed;
            rot = faceLeft;
        }
        if (Input.GetKey(RightKey))
        {
            dir -= transform.right * speed;
            rot = faceRight;
        }

        if(Input.GetKey(ForwardKey) && Input.GetKey(LeftKey))
        {
            rot = faceLeft * 0.5f;
        }
        if (Input.GetKey(ForwardKey) && Input.GetKey(RightKey))
        {
            rot = faceRight * 0.5f;
        }
        if (Input.GetKey(BackKey) && Input.GetKey(LeftKey))
        {
            rot = faceLeft * 1.5f;
        }
        if (Input.GetKey(BackKey) && Input.GetKey(RightKey))
        {
            rot = faceRight * 1.5f;
        }

        //Move
        transform.Translate(dir);

        //Rotation
        Quaternion targetRot = Quaternion.AngleAxis(rot.y, Vector3.up);
        transform.rotation = Quaternion.Slerp(transform.rotation, targetRot, 1f);
    }
}
