﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

[System.Serializable]
public class SignalEvent : UnityEvent{ }

public class Wifi : MonoBehaviour
{
    const float SIGNAL_ONE_MULTIPLIER = 0.5f;
    const float SIGNAL_TWO_MULTIPLIER = 0.75f;
    const float SIGNAL_THREE_MULTIPLIER = 1.00f;
    const float SIGNAL_FOUR_MULTIPLIER = 1.25f;
    const float SIGNAL_FIVE_MULTIPLIER = 1.5f;

    public TutorialHandler _TutorialHandler;
    public float wifiRadius = 10.0f; //Default is 10
    public float DownloadSpeed = 8.0f;//Default is 10
    public SignalEvent OnFirstBarEnter;
    public SignalEvent OnSecondBarEnter;
    public SignalEvent OnThirdBarEnter;
    public SignalEvent OnForthBarEnter;
    public SignalEvent OnFifthBarEnter;
    public int Index = 0;

    private Transform target;
    private float playerDistance;
    private readonly float interval = 2f;
    private GameManager gameManager;
    
    // Start is called before the first frame update
    void Start()
    {
        target = PlayerController.Instance.transform;
        gameManager = GameManager.Instance;
    }

    // Update is called once per frame
    void Update()
    {
        if(gameManager && gameManager.CurState == GameState.Start && this.enabled == true)
        {
            CheckDistance();
        }
    }

    void OnDrawGizmosSelected()
    {
        Gizmos.color = Color.red;
        Gizmos.DrawWireSphere(this.transform.position, wifiRadius * SIGNAL_ONE_MULTIPLIER);
        Gizmos.color = Color.red;
        Gizmos.DrawWireSphere(this.transform.position, wifiRadius * SIGNAL_TWO_MULTIPLIER);
        Gizmos.color = Color.yellow;
        Gizmos.DrawWireSphere(this.transform.position, wifiRadius * SIGNAL_THREE_MULTIPLIER);
        Gizmos.color = Color.yellow;
        Gizmos.DrawWireSphere(this.transform.position, wifiRadius * SIGNAL_FOUR_MULTIPLIER);
        Gizmos.color = Color.green;
        Gizmos.DrawWireSphere(this.transform.position, wifiRadius * SIGNAL_FIVE_MULTIPLIER);
    }

    void CheckDistance() // Normalize (*To Follow)
    {
        playerDistance = Vector3.Distance(transform.position, target.position);

        bool called = false;

        if (playerDistance < wifiRadius * SIGNAL_FIVE_MULTIPLIER &&
            playerDistance > wifiRadius * SIGNAL_FOUR_MULTIPLIER)
        {
            Debug.Log("First Bar");
            called = true;
            Index = 1;
            OnFirstBarEnter.Invoke();
            if(_TutorialHandler != null) OnFirstBarEnter.RemoveAllListeners();
        }
        if (playerDistance < wifiRadius * SIGNAL_FOUR_MULTIPLIER &&
            playerDistance > wifiRadius * SIGNAL_THREE_MULTIPLIER)
        {
            Debug.Log("Second Bar");
            called = true;
            Index = 2;
            OnSecondBarEnter.Invoke();
            if (_TutorialHandler != null) OnSecondBarEnter.RemoveAllListeners();
        }
        if (playerDistance < wifiRadius * SIGNAL_THREE_MULTIPLIER &&
            playerDistance > wifiRadius * SIGNAL_TWO_MULTIPLIER)
        {
            Debug.Log("Third Bar");
            called = true;
            Index = 3;
            OnThirdBarEnter.Invoke();
            if (_TutorialHandler != null) OnThirdBarEnter.RemoveAllListeners();
        }
        if (playerDistance < wifiRadius * SIGNAL_TWO_MULTIPLIER &&
            playerDistance > wifiRadius * SIGNAL_ONE_MULTIPLIER)
        {
            Debug.Log("Forth Bar");
            called = true;
            Index = 4;
            OnForthBarEnter.Invoke();
            if (_TutorialHandler != null) OnForthBarEnter.RemoveAllListeners();
        }
        if (playerDistance < wifiRadius * SIGNAL_ONE_MULTIPLIER)
        {
            Debug.Log("Fifth Bar");
            called = true;
            Index = 5;
            OnFifthBarEnter.Invoke();
            if (_TutorialHandler != null) OnFifthBarEnter.RemoveAllListeners();
        }

        if (called == false)
        {
            Index = 0;
            OnThirdBarEnter.Invoke();
        }
    }

    public IEnumerator StartDownload(File file)
    {
        while (true)
        {
            float derivedSpeed = GetDerivedDownloadSpeed();
            Debug.Log("Download Speed: " + derivedSpeed);
            file.Download(derivedSpeed);

            if (file.CurFileState == FileState.FinishDownload) yield break;
            yield return new WaitForSeconds(1.0f);
        }
    }

    float GetDerivedDownloadSpeed()
    {
        return Random.Range(DownloadSpeed - interval, DownloadSpeed + interval);
    }
}
